@@ -14,7 +14,7 @@
   OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */
 
-#include "suil_internal.h"
+#include "./suil_internal.h"
 
 SUIL_API
 SuilHost*
@@ -28,10 +28,19 @@ suil_host_new(SuilPortWriteFunc       write_func,
 	host->index_func       = index_func;
 	host->subscribe_func   = subscribe_func;
 	host->unsubscribe_func = unsubscribe_func;
+	host->touch_func       = NULL;
 	host->gtk_lib          = NULL;
 	return host;
 }
 
+SUIL_API
+void
+suil_host_set_touch_func(SuilHost*     host,
+                         SuilTouchFunc touch_func)
+{
+	host->touch_func = touch_func;
+}
+
 SUIL_API
 void
 suil_host_free(SuilHost* host)
