@@ -1,5 +1,5 @@
 /*
-  Copyright 2007-2011 David Robillard <http://drobilla.net>
+  Copyright 2007-2012 David Robillard <http://drobilla.net>
 
   Permission to use, copy, modify, and/or distribute this software for any
   purpose with or without fee is hereby granted, provided that the above
@@ -20,9 +20,7 @@
 #include <assert.h>
 #include <stdlib.h>
 
-#include "lv2/lv2plug.in/ns/extensions/ui/ui.h"
-
-#ifdef __WIN32__
+#ifdef _WIN32
 #include <windows.h>
 #define dlopen(path, flags) LoadLibrary(path)
 #define dlclose(lib) FreeLibrary(lib)
@@ -32,6 +30,8 @@ static inline char* dlerror(void) { return "Unknown error"; }
 #include <dlfcn.h>
 #endif
 
+#include "lv2/lv2plug.in/ns/extensions/ui/ui.h"
+
 #include "suil/suil.h"
 
 #define SUIL_ERRORF(fmt, ...) fprintf(stderr, "error: %s: " fmt, \
@@ -42,32 +42,50 @@ struct SuilHostImpl {
 	SuilPortIndexFunc       index_func;
 	SuilPortSubscribeFunc   subscribe_func;
 	SuilPortUnsubscribeFunc unsubscribe_func;
+	SuilTouchFunc           touch_func;
 	void*                   gtk_lib;
 };
 
+struct _SuilWrapper;
+
+typedef void (*SuilWrapperFreeFunc)(struct _SuilWrapper*);
+
+typedef int (*SuilWrapperWrapFunc)(struct _SuilWrapper* wrapper,
+                                   SuilInstance*        instance);
+
+typedef struct _SuilWrapper {
+	SuilWrapperWrapFunc wrap;
+	SuilWrapperFreeFunc free;
+	void*               lib;
+	void*               impl;
+	LV2UI_Resize        resize;
+} SuilWrapper;
+
 struct SuilInstanceImpl {
 	void*                   lib_handle;
 	const LV2UI_Descriptor* descriptor;
 	LV2UI_Handle            handle;
+	SuilWrapper*            wrapper;
+	LV2_Feature**           features;
+	LV2UI_Port_Map          port_map;
+	LV2UI_Port_Subscribe    port_subscribe;
+	LV2UI_Touch             touch;
 	SuilWidget              ui_widget;
 	SuilWidget              host_widget;
 };
 
 /**
-   Type of a module's suil_wrap_init function.
+   The type of the suil_wrapper_new entry point in a wrapper module.
 
-   This initialisation function must be called before instantiating any UI that
-   will need to be wrapped by this wrapper (e.g. it will perform any
-   initialisation required to create a widget for the given toolkit).
+   This constructs a SuilWrapper which contains everything necessary
+   to wrap a widget, including a possibly extended features array to
+   be used for instantiating the UI.
 */
-typedef int (*SuilWrapInitFunc)(SuilHost*                 host,
-                                const char*               host_type_uri,
-                                const char*               ui_type_uri,
-                                const LV2_Feature* const* features);
-
-typedef int (*SuilWrapFunc)(const char*   host_type_uri,
-                            const char*   ui_type_uri,
-                            SuilInstance* instance);
+typedef SuilWrapper* (*SuilWrapperNewFunc)(SuilHost*      host,
+                                           const char*    host_type_uri,
+                                           const char*    ui_type_uri,
+                                           LV2_Feature*** features,
+                                           unsigned       n_features);
 
 typedef void (*SuilVoidFunc)();
 
@@ -80,4 +98,20 @@ suil_dlfunc(void* handle, const char* symbol)
 	return dlfunc(handle, symbol);
 }
 
+/** Add a feature to a (mutable) LV2 feature array. */
+static inline void
+suil_add_feature(LV2_Feature*** features,
+                 unsigned       n,
+                 const char*    uri,
+                 void*          data)
+{
+	*features = (LV2_Feature**)realloc(*features,
+	                                   sizeof(LV2_Feature*) * (n + 2));
+
+	(*features)[n]       = (LV2_Feature*)malloc(sizeof(LV2_Feature));
+	(*features)[n]->URI  = uri;
+	(*features)[n]->data = data;
+	(*features)[n + 1]   = NULL;
+}
+
 #endif  // SUIL_INTERNAL_H
