@@ -49,7 +49,15 @@ extern "C" {
 
 /**
    @defgroup suil Suil
-   A library for hosting LV2 plugin UIs.
+
+   Suil is a library for loading and wrapping LV2 plugin UIs.  With Suil, a
+   host written in one supported toolkit can embed a plugin UI written in a
+   different supported toolkit.  Suil insulates hosts from toolkit libraries
+   used by plugin UIs.  For example, a Gtk host can embed a Qt UI without
+   linking against Qt at compile time.
+
+   Visit <http://drobilla.net/software/suil> for more information.
+
    @{
 */
 
@@ -149,7 +157,7 @@ suil_host_free(SuilHost* host);
    @param ui_type_uri The URI of the UI widget type.
    @return 0 if wrapping is unsupported, otherwise the quality of the wrapping
    where 1 is the highest quality (direct native embedding with no wrapping)
-   and increaing values are of a progressively lower quality and/or stability.
+   and increasing values are of a progressively lower quality and/or stability.
 */
 SUIL_API
 unsigned
@@ -158,6 +166,12 @@ suil_ui_supported(const char* host_type_uri,
 
 /**
    Instantiate a UI for an LV2 plugin.
+
+   This funcion may load a suil module to adapt the UI to the desired toolkit.
+   Suil is configured at compile time to load modules from the appropriate
+   place, but this can be changed at run-time via the environment variable
+   SUIL_MODULE_DIR.  This makes it possible to bundle suil with an application.
+
    @param host Host descriptor.
    @param controller Opaque host controller pointer.
    @param container_type_uri URI of the desired host container widget type.
@@ -205,6 +219,19 @@ suil_instance_get_widget(SuilInstance* instance);
 
 /**
    Notify the UI about a change in a plugin port.
+   @param instance UI instance.
+   @param port_index Index of the port which has changed.
+   @param buffer_size Size of @p buffer in bytes.
+   @param format Format of @p buffer (mapped URI, or 0 for float).
+   @param buffer Change data, e.g. the new port value.
+
+   This function can be used to notify the UI about any port change, but in the
+   simplest case is used to set the value of lv2:ControlPort ports.  For
+   simplicity, this is a special case where @p format is 0, @p buffer_size is
+   4, and @p buffer should point to a single float.
+
+   The @p buffer must be valid only for the duration of this call, the UI must
+   not keep a reference to it.
 */
 SUIL_API
 void
